package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.constants.DefaultMenuConstants;
import cn.com.ty.lift.common.utils.ProjectUtils;
import cn.com.ty.lift.system.user.dao.entity.Menu;
import cn.com.ty.lift.system.user.dao.entity.Role;
import cn.com.ty.lift.system.user.dao.entity.RoleMenu;
import cn.com.ty.lift.system.user.dao.entity.UserRole;
import cn.com.ty.lift.system.user.dao.entity.model.RoleRequest;
import cn.com.ty.lift.system.user.dao.entity.model.RoleResponse;
import cn.com.ty.lift.system.user.dao.mapper.RoleMapper;
import cn.com.ty.lift.system.user.service.IMenuService;
import cn.com.ty.lift.system.user.service.IRoleMenuService;
import cn.com.ty.lift.system.user.service.IRoleService;
import cn.com.ty.lift.system.user.service.IUserRoleService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author huangyuan
 * @date 2019-12-01
 * @description 角色接口实现类
 */
@Service
public class RoleService extends ServiceImpl<RoleMapper, Role> implements IRoleService {

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private IRoleMenuService roleMenuService;

    @Autowired
    private IUserRoleService userRoleService;

    @Autowired
    private IMenuService menuService;

    @Override
    public RestResponse list(RoleRequest roleRequest) {
        IPage<Role> rolePage = new Page<>(roleRequest.getPageNum(), roleRequest.getPageSize());
        rolePage = this.page(rolePage, new QueryWrapper<Role>()
                .eq("delete_flag", ApiConstants.DELETE_NO)
        );
        return RestResponse.ok(rolePage, ApiConstants.RESULT_SUCCESS, "获取角色列表成功");
    }

    @Override
    @Transactional
    public RestResponse addRole(RoleRequest roleRequest) {
        Role role = new Role(roleRequest);
        role.setCreateTime(new Date());
        //保存角色信息
        int insertCount = roleMapper.insert(role);
        //保存角色菜单关联关系
        boolean saveBatchFlag = roleMenuService.saveRoleMenu(role.getId(), roleRequest.getMenuIds());
        if (insertCount == 0 || !saveBatchFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "保存角色信息失败");
        }
        return RestResponse.ok(role, ApiConstants.RESULT_SUCCESS, "保存角色信息成功");
    }

    @Override
    @Transactional
    public RestResponse updateRole(RoleRequest roleRequest) {
        Role role = new Role(roleRequest);
        role.setUpdateTime(new Date());
        //更新角色信息
        int updateCount = roleMapper.updateById(role);
        //删除角色原来关联的菜单信息
        boolean deleteFlag = roleMenuService.deleteRoleMenu(role.getId());
        //保存角色关联的新的菜单信息
        boolean saveBatchFlag = roleMenuService.saveRoleMenu(role.getId(), roleRequest.getMenuIds());
        if (updateCount == 0 || !deleteFlag || !saveBatchFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "更新角色信息失败");
        }
        return RestResponse.ok(role, ApiConstants.RESULT_SUCCESS, "更新角色信息成功");
    }

    @Override
    @Transactional
    public RestResponse deleteRole(RoleRequest roleRequest) {
        Long roleId = roleRequest.getId();
        //判断角色有没有分配给用户
        List<UserRole> userRoleList = userRoleService.getByRoleId(roleId);
        if (userRoleList != null && userRoleList.size() > 0) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "角色已关联用户，无法删除");
        }
        Role role = roleMapper.selectById(roleId);
        role.setDeleteFlag(ApiConstants.DELETE_YES);
        int updateCount = roleMapper.updateById(role);
        //删除角色菜单关联信息
        boolean deleteFlag = roleMenuService.deleteRoleMenu(roleId);
        if (updateCount == 0 || !deleteFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "删除失败");
        }
        return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "删除成功");
    }

    @Override
    public RestResponse getUserMenus(UserRole userRole) {
        userRole = userRoleService.getByUserRole(userRole);
        String menuUrl = "";
        if (userRole != null) {
            List<Menu> menuList = menuService.getByRoleId(userRole.getRoleId());
            menuUrl = menuService.menuToUrl(menuList);
        }
        return RestResponse.ok(menuUrl, ApiConstants.RESULT_SUCCESS, "获取菜单信息成功");
    }

    @Override
    public Role getRoleByUserIdAndCompanyId(Long userId, Long companyId) {
        UserRole userRole = userRoleService.getOne(new QueryWrapper<UserRole>()
                .eq("user_id", userId)
                .eq("company_id", companyId)
        );
        if (userRole != null) {
            return getById(userRole.getRoleId());
        }
        return null;
    }

    @Override
    @Transactional
    public void saveDefaultMessage(Long companyId) {
        //默认角色列表
        List<Role> defaultRoleList = new ArrayList<>();
        for (CommonEnum.DefaultRole defaultRole : DefaultMenuConstants.DEFAULT_ROLE) {
            Role newRole = new Role(defaultRole);
            newRole.setCompanyId(companyId);
            newRole.setDescription(defaultRole.getLabel());
            defaultRoleList.add(newRole);
        }
        //保存默认角色
        this.saveBatch(defaultRoleList);
        //关联关系列表
        List<RoleMenu> needSaveRoleMenuList = new ArrayList<>();
        //生成角色菜单关联关系
        for (Role role : defaultRoleList) {
            Long[] menuIds = DefaultMenuConstants.DEFAULT_ROLE_MENUIDS.get(role.getCode());
            for (Long menuId : menuIds) {
                RoleMenu roleMenu = new RoleMenu();
                roleMenu.setMenuId(menuId);
                roleMenu.setRoleId(role.getId());
                needSaveRoleMenuList.add(roleMenu);
            }
        }
        //需要保存角色菜单信息
        roleMenuService.saveBatch(needSaveRoleMenuList);
    }

    @Override
    public Map<Long, Role> getUserIdToRoleByUserIdsAndCompanyId(List<Long> userIdList, Long companyId) {
        Map<Long, Role> userIdToRole = new HashMap<>();
        if(userIdList != null && userIdList.size() > 0){
            List<UserRole> userRoleList = userRoleService.list(new QueryWrapper<UserRole>()
                    .in("user_id", userIdList)
                    .eq("company_id", companyId)
            );
            //转化用户id对应用户角色关联关系
            Map<Long, UserRole> userIdToUserRole = ProjectUtils.attrToObjMap(userRoleList, "userId", null);
            //获取角色列表
            List<Long> roleIdList = ProjectUtils.getAttrList(userRoleList, "roleId", null);
            List<Role> roleList = (List<Role>) this.listByIds(roleIdList);
            Map<Long, Role> idToRole = ProjectUtils.attrToObjMap(roleList, "id", null);
            for(Long userId: userIdList){
                UserRole userRole = userIdToUserRole.get(userId);
                if(userRole != null){
                    Role role = idToRole.get(userRole.getRoleId());
                    userIdToRole.put(userId, role);
                }
            }
        }
        return userIdToRole;
    }

    @Override
    public RestResponse info(RoleRequest roleRequest) {
        Long roleId = roleRequest.getId();
        RoleResponse roleResponse = new RoleResponse();
        //获取角色
        Role role = this.getById(roleId);
        if(role != null){
            //角色基础信息查看
            roleResponse.setId(roleId);
            roleResponse.setName(role.getName());
            roleResponse.setId(role.getId());
            //获取角色菜单信息
            List<Menu> roleMenuList = roleMenuService.getMenuTree(roleId);
            roleResponse.setCompanyMenuList(roleMenuList);
        }
        return RestResponse.ok(roleResponse, ApiConstants.RESULT_SUCCESS, "获取角色信息成功");
    }
}
